unit uDBConfiguration;

interface

uses
  Windows, Dialogs, StrUtils, DateUtils, Math, Forms, Messages, SysUtils,
  Classes, WideStrings, DBXFirebird, DB, DBClient, SimpleDS, SqlExpr,
  Controls, ExtCtrls, ComCtrls, uReceita;

type
  TDBConfig = class
  private
    oSQLConnection: TSQLConnection;
    oDataSet: TSimpleDataSet;

    constructor Create;
    destructor Destroy;
  public
    procedure ConfigureSQLConn(aSQLConnection: TSQLConnection);
    procedure SetSQLConn(aSQLConnection: TSQLConnection);
    function SaveIngrediente(aIngrediente, aUnidade: string; aPreco: Double): Boolean;
    function SearchIngredientes(aDataSet : TSimpleDataSet; aBusca : string = '') : Boolean;
    function AlterIngrediente(aId : Integer; aIngrediente, aUnidade: string; aPreco: Double) : Boolean;
    function SaveResponsavel(aNome, aSenha, aTel, aEmail, aLogin: string; aIsUser: Boolean): Boolean;
    function SearchResp(aDataSet : TSimpleDataSet; aBusca : string = '') : Boolean;
    function AlterReponsavel(aId : Integer; aNome, aSenha, aTel, aEmail, aLogin: string; aIsUser: Boolean) : Boolean;
    procedure SearchIngForCombo(aLstIngredientes, aLstUnidades, aLstID : TStringList);
    procedure SearchRespForCombo(aLstResponsaveis, oLstId : TStringList);
    function SaveReceita(aListIng : TListView; aTitulo, aRendimento, aResponsavel, aCategoria, aPreparo, aObs : string) : Boolean;
    function DeleteReceita(aID : Integer) : Boolean;
    function UpdateReceita(aListIng : TListView; aTitulo, aRendimento, aResponsavel, aCategoria, aPreparo, aObs : string; aID : Integer) : Boolean;

    function CarregaReceitas(aReceitaMain : TReceitaMain) : Boolean;

    class function GetInstance: TDBConfig;
    class procedure DestroyInstance;
  end;

implementation

var
  gDBConfig: TDBConfig = nil;

  { TDBConfig }

function TDBConfig.AlterIngrediente(aId: Integer; aIngrediente, aUnidade: string; aPreco: Double): Boolean;
var
  vQry: string;
begin
  Result := False;

  vQry := 'Update Ingrediente set Nome = '+ QuotedStr(aIngrediente)+ ', ' +
          'Unidade = ' + QuotedStr(aUnidade) + ', ' +
          'Preco = ' + StringReplace(FormatFloat('0.00', aPreco), ',', '.', []) +
          ' where ID = ' + IntToStr(aID);

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel alterar Ingrediente.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  Result := True;
end;

function TDBConfig.AlterReponsavel(aId: Integer; aNome, aSenha, aTel, aEmail, aLogin: string; aIsUser: Boolean): Boolean;
var
  vQry: string;
begin
  Result := False;

  vQry := 'Update Responsavel set Nome = '+ QuotedStr(aNome)+ ', ' +
          'Telefone = ' + QuotedStr(aTel) + ', ' +
          'Email = ' + QuotedStr(aEmail) + ', ' +
          'Login = ' + QuotedStr(aLogin) + ', ' +
          'Senha = ' + QuotedStr(aSenha) + ', ' +
          'IsUser = ' + QuotedStr(IfThen(aIsUser, '1', '0')) +
          ' where ID = ' + IntToStr(aID);

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel alterar Responsvel.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  Result := True;
end;

function TDBConfig.CarregaReceitas(aReceitaMain: TReceitaMain): Boolean;
var
  vReceita : TReceita;
  vIngrediente : TIngrediente;
  vIndexReceita : Integer;
  vIndexIng : Integer;
begin
  Result := False;

  aReceitaMain.Clear;

  oDataSet.Active := False;
  oDataSet.DataSet.CommandText := 'Select R.ID, R.Descricao, R.DtCadastro, R.Quantidade, R.Categoria, R.Preparo, '#13#10+
                                  '       R.OBS, Re.Nome as Resp, I.Nome, I.Unidade, RF.Quantidade as Qtde, '+
                                  '       I.ID as IngID from ReceitaFinal RF '#13#10+
                                  'Join Ingrediente I on I.id = RF.ingredientes_id '#13#10+
                                  'Join Receita R on R.id = RF.receita_id '#13#10+
                                  'Join Responsavel Re on Re.id = R.responsavel '#13#10+
                                  'Order By R.id';
  oDataSet.Active := True;

  oDataSet.First;
  while (not oDataSet.Eof) do begin
    vIndexReceita := aReceitaMain.IndexOfReceita(oDataSet.Fields.FieldByName('ID').AsInteger);

    if (vIndexReceita = -1) then begin
      vReceita := TReceita.Create;
      vReceita.ID := oDataSet.Fields.FieldByName('ID').AsInteger;
      vReceita.Titulo := oDataSet.Fields.FieldByName('Descricao').AsString;
      vReceita.DtCadastro := oDataSet.Fields.FieldByName('DtCadastro').AsDateTime;
      vReceita.Responsavel := oDataSet.Fields.FieldByName('Resp').AsString;
      vReceita.Rendimeneto := oDataSet.Fields.FieldByName('Quantidade').AsInteger;;
      vReceita.Categoria := oDataSet.Fields.FieldByName('Categoria').AsString;
      vReceita.Preparo := oDataSet.Fields.FieldByName('Preparo').AsString;
      vReceita.Obs := oDataSet.Fields.FieldByName('Obs').AsString;
      aReceitaMain.ListReceitas.Add(vReceita);
      vIndexReceita := aReceitaMain.ListReceitas.Count - 1;
    end;

    vIndexIng := TReceita(aReceitaMain.ListReceitas[vIndexReceita]).IndexOfIngrediente(oDataSet.Fields.FieldByName('Nome').AsString);
    if (vIndexIng = -1) then begin
      vIngrediente := TIngrediente.Create;
      vIngrediente.Descricao := oDataSet.Fields.FieldByName('Nome').AsString;
      vIngrediente.Unidade := oDataSet.Fields.FieldByName('Unidade').AsString;
      vIngrediente.Quantidade := oDataSet.Fields.FieldByName('Qtde').AsFloat;
      vIngrediente.ID := oDataSet.Fields.FieldByName('IngID').AsInteger;
      TReceita(aReceitaMain.ListReceitas[vIndexReceita]).LstIngrediente.Add(vIngrediente);
    end
    else begin
      TIngrediente(TReceita(aReceitaMain.ListReceitas[vIndexReceita]).LstIngrediente[vIndexIng]).Quantidade :=
      TIngrediente(TReceita(aReceitaMain.ListReceitas[vIndexReceita]).LstIngrediente[vIndexIng]).Quantidade +
      oDataSet.Fields.FieldByName('Qtde').AsFloat;
    end;

    oDataSet.Next;
  end;

  Result := True;
end;

procedure TDBConfig.ConfigureSQLConn(aSQLConnection: TSQLConnection);
var
  vDBPath: string;
begin
  vDBPath := ExtractFilePath(Application.ExeName);
  vDBPath := StringReplace(vDBPath, 'Bin\', 'Data\RECEITA.gdb',
    [rfReplaceAll, rfIgnoreCase]);

  aSQLConnection.LoadParamsOnConnect := False;
  aSQLConnection.Connected := False;
  aSQLConnection.Params.Clear;

  aSQLConnection.Params.Add('DriverName=Firebird');
  aSQLConnection.Params.Add('Database=' + vDBPath);
  aSQLConnection.Params.Add('RoleName=RoleName');
  aSQLConnection.Params.Add('User_Name=SYSDBA');
  aSQLConnection.Params.Add('Password=masterkey');
  aSQLConnection.Params.Add('ServerCharSet=');
  aSQLConnection.Params.Add('SQLDialect=3');
  aSQLConnection.Params.Add('BlobSize=-1');
  aSQLConnection.Params.Add('CommitRetain=False');
  aSQLConnection.Params.Add('ErrorResourceFile=');
  aSQLConnection.Params.Add('WaitOnLocks=True');
  aSQLConnection.Params.Add('LocaleCode=0000');
  aSQLConnection.Params.Add('Interbase TransIsolation=ReadCommited');
  aSQLConnection.Params.Add('Trim Char=False');

  aSQLConnection.Connected := True;
end;

constructor TDBConfig.Create;
begin
  oDataSet := TSimpleDataSet.Create(nil);
end;

function TDBConfig.DeleteReceita(aID: Integer): Boolean;
var
  vQry: string;
begin
  Result := False;

  vQry := 'Delete from RECEITAFINAL where RECEITA_ID = ' + IntToStr(aID);

  oSQLConnection.Execute(vQry, oDataSet.Params);

  vQry :='Delete from Receita where ID = ' + IntToStr(aID);

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel realizar a excluso da Receita.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  Result := True;
end;

destructor TDBConfig.Destroy;
begin
  if Assigned(oDataSet) then
  begin
    oDataSet.Destroy;
  end;
end;

class procedure TDBConfig.DestroyInstance;
begin
  gDBConfig.Destroy;
end;

class function TDBConfig.GetInstance: TDBConfig;
begin
  if (not Assigned(gDBConfig)) then
  begin
    gDBConfig := TDBConfig.Create;
  end;

  Result := gDBConfig;
end;

function TDBConfig.SaveIngrediente(aIngrediente, aUnidade: string; aPreco: Double): Boolean;
var
  vQry: string;
begin
  Result := False;

  vQry := 'Insert into Ingrediente (Nome, Unidade, Preco) values ('+
          QuotedStr(aIngrediente) + ', ' +
          QuotedStr(aUnidade) + ', ' +
          StringReplace(FormatFloat('0.00', aPreco), ',', '.', []) + ')';

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel realizar a gravao do Ingrediente.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  Result := True;
end;

function TDBConfig.SaveReceita(aListIng: TListView; aTitulo, aRendimento, aResponsavel, aCategoria, aPreparo, aObs: string): Boolean;
var
  vQry: string;
  vIdReceita : Integer;
  I : Integer;
  vQuantidade : Double;
begin
  Result := False;

  vQry := 'Insert into Receita (Descricao, DtCadastro, Quantidade, Categoria, Preparo, Obs, Responsavel) values ('+
          QuotedStr(aTitulo) + ', ' +
          QuotedStr(FormatDateTime('yyyy-mm-dd', Now)) + ', ' +
          aRendimento+ ', ' +
          QuotedStr(aCategoria) + ', ' +
          QuotedStr(aPreparo) + ', ' +
          QuotedStr(aObs) + ', ' +
          aResponsavel + ')';

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel realizar a gravao da Receita.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  oDataSet.Active := False;
  oDataSet.DataSet.CommandText := 'Select ID from Receita order By Id desc';
  oDataSet.Active := True;

  vIdReceita := oDataSet.Fields.FieldByName('ID').AsInteger;

  vQry := EmptyStr;
  for I := 0 to aListIng.Items.Count - 1 do begin
    vQuantidade := StrToFloatDef(aListIng.Items[I].Caption, 0);
    vQry := 'Insert into ReceitaFinal (INGREDIENTES_ID, RECEITA_ID, QUANTIDADE) '+
            'values ('+ aListIng.Items[I].SubItems[2] + ', '+
            IntToStr(vIdReceita) + ', ' +
            StringReplace(FormatFloat('0.00', vQuantidade), ',', '.', []) + ')';

    if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
    begin
      MessageBox(0, PChar('No foi possvel realizar a gravao da Receita.'),
      PChar(Application.Title), MB_OK + MB_ICONERROR);
      Exit;
    end;
  end;

  Result := True;
end;

function TDBConfig.SaveResponsavel(aNome, aSenha, aTel, aEmail, aLogin: string; aIsUser: Boolean): Boolean;
var
  vQry: string;
begin
  Result := False;

  vQry := 'Insert into Responsavel (Nome, Telefone, Email, Login, Senha, DtCadastro, IsUser) values ('+
          QuotedStr(aNome) + ', ' +
          QuotedStr(aTel) + ', ' +
          QuotedStr(aEmail) + ', ' +
          QuotedStr(aLogin) + ', ' +
          QuotedStr(aSenha) + ', ' +
          QuotedStr(FormatDateTime('yyyy-mm-dd', Now)) + ', ' +
          QuotedStr(IfThen(aIsUser, '1', '0'))+ ')';

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel realizar a gravao do Responsvel.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  Result := True;
end;

procedure TDBConfig.SearchIngForCombo(aLstIngredientes, aLstUnidades, aLstID: TStringList);
begin
  oDataSet.Active := False;
  oDataSet.DataSet.CommandText := 'Select ID, Nome, Unidade from Ingrediente order by Nome';
  oDataSet.Active := True;

  oDataSet.First;
  while not oDataSet.Eof do begin
    aLstIngredientes.Add(oDataSet.Fields.FieldByName('Nome').AsString);
    aLstUnidades.Add(oDataSet.Fields.FieldByName('Unidade').AsString);
    aLstID.Add(oDataSet.Fields.FieldByName('ID').AsString);
    oDataSet.Next;
  end;
end;

function TDBConfig.SearchIngredientes(aDataSet: TSimpleDataSet; aBusca : string = ''): Boolean;
var
  vQry : string;
begin
  Result := False;

  if (aBusca <> EmptyStr) then begin
    vQry := 'Select * from Ingrediente where Nome like '+ QuotedStr('%' + aBusca + '%');
  end
  else begin
    vQry := 'Select * from Ingrediente';
  end;

  aDataSet.Active := False;
  aDataSet.Connection := oSQLConnection;

  aDataSet.DataSet.CommandText := vQry;
  aDataSet.Active := True;

  Result := True;
end;

function TDBConfig.SearchResp(aDataSet: TSimpleDataSet; aBusca: string): Boolean;
var
  vQry : string;
begin
  Result := False;

  if (aBusca <> EmptyStr) then begin
    vQry := 'Select * from Responsavel where Nome like '+ QuotedStr('%' + aBusca + '%');
  end
  else begin
    vQry := 'Select * from Responsavel';
  end;

  aDataSet.Active := False;
  aDataSet.Connection := oSQLConnection;

  aDataSet.DataSet.CommandText := vQry;
  aDataSet.Active := True;

  Result := True;
end;

procedure TDBConfig.SearchRespForCombo(aLstResponsaveis, oLstId: TStringList);
begin
  oDataSet.Active := False;
  oDataSet.DataSet.CommandText := 'Select ID, Nome from Responsavel order by Nome';
  oDataSet.Active := True;

  oDataSet.First;
  while not oDataSet.Eof do begin
    aLstResponsaveis.Add(oDataSet.Fields.FieldByName('Nome').AsString);
    oLstId.Add(oDataSet.Fields.FieldByName('ID').AsString);
    oDataSet.Next;
  end;
end;

procedure TDBConfig.SetSQLConn(aSQLConnection: TSQLConnection);
begin
  oSQLConnection := aSQLConnection;
  oDataSet.Connection := oSQLConnection;
end;

function TDBConfig.UpdateReceita(aListIng: TListView; aTitulo, aRendimento, aResponsavel, aCategoria, aPreparo, aObs: string; aID: Integer): Boolean;
var
  vQuantidade : Double;
  vQry : string;
  I : integer;
begin

  vQry := 'Delete from ReceitaFinal where Receita_ID = ' + IntToStr(aID);

  oSQLConnection.Execute(vQry, oDataSet.Params);

  vQry := 'Update Receita Set Descricao = '+ QuotedStr(aObs) +
          ', Quantidade = '+ QuotedStr(aRendimento) +
          ', Categoria = '+ QuotedStr(aCategoria) +
          ', Preparo = '+ QuotedStr(aPreparo) +
          ', Obs = '+ QuotedStr(aObs) +
          ', Responsavel = '+ QuotedStr(aResponsavel) +
          ' where ID = '+ IntToStr(aID);

  if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
  begin
    MessageBox(0, PChar('No foi possvel realizar a gravao da Receita.'),
    PChar(Application.Title), MB_OK + MB_ICONERROR);
    Exit;
  end;

  vQry := EmptyStr;
  for I := 0 to aListIng.Items.Count - 1 do begin
    vQuantidade := StrToFloatDef(aListIng.Items[I].Caption, 0);
    vQry := 'Insert into ReceitaFinal (INGREDIENTES_ID, RECEITA_ID, QUANTIDADE) '+
            'values ('+ aListIng.Items[I].SubItems[2] + ', '+
            IntToStr(aID) + ', ' +
            StringReplace(FormatFloat('0.00', vQuantidade), ',', '.', []) + ')';

    if (oSQLConnection.Execute(vQry, oDataSet.Params) <> 1) then
    begin
      MessageBox(0, PChar('No foi possvel realizar a gravao da Receita.'),
      PChar(Application.Title), MB_OK + MB_ICONERROR);
      Exit;
    end;
  end;

  Result := True;
end;

end.
